==> openGion_dev_5.3.4.0.zip: opengion/uap/webapps/gf/src/org/opengion/plugin/query/Query_JDBCArrayCallable.java <==
/*
 * Copyright (c) 2009 The openGion Project.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.opengion.plugin.query;

import org.opengion.hayabusa.common.HybsSystem;
import org.opengion.hayabusa.common.HybsSystemException;
import org.opengion.hayabusa.db.AbstractQuery;
import org.opengion.hayabusa.db.DBErrMsg;
import org.opengion.fukurou.util.ErrorMessage;
import org.opengion.fukurou.util.StringUtil;
import org.opengion.fukurou.util.Closer;

import java.sql.CallableStatement;
import java.sql.Connection;
import java.sql.SQLException;
import java.sql.Types;
import java.util.Map;

import oracle.jdbc.OracleCallableStatement;
import oracle.jdbc.OracleTypes;
import oracle.sql.ARRAY;
// import oracle.sql.ArrayDescriptor;

/**
 * Callableのエラー配列対応版です。バッチ系標準のPL/SQL をコールする Query クラスです。
 *
 * java.sql.CallableStatement を用いて、データベース検索処理を行います。
 * 引数は、従来のPL/SQLの実行が可能なように、第一引数はエラーコード、第二引数は、
 * エラーメッセージを返してきます。第三引数以降は、自由に指定できます。
 * 内部変数の受け渡しのデフォルト実装は、AbstractQuery クラスを継承している
 * ため,ここでは、execute() メソッドを実装しています。
 *
 * @og.formSample
 * 例：
 *     第一引数、第二引数は、通常のPL/SQLと同じ、結果(STATUS)と
 *     内容(ERR_CODE)を返します。
 *     それ以降の引数については、入力（IN)のみですが、自由に設定できます。
 *     引数に変数を使用する場合は、? 記号を当てはめます。
 *     第一引数、第二引数は、予約済みですが、それ以降は、好きな位置に割り当てられます。
 *     names 属性の順番に、値だけがセットされていきます。
 *     下記の例は、変数の引数は、使用していません。
 *
 * &lt;h:query
 *     command=&quot;NEW&quot;
 *     queryType=&quot;JDBCArrayCallable&quot;
 *     displayMsg=&quot;&quot; &gt;
 *     &lt;jsp:text&gt;
 *         { call GEP00002.GEP00002( ?,?,&#39;{&#064;GUI.KEY}&#39;,&#39;{&#064;USER.ID}&#39; ) }
 *     &lt;/jsp:text&gt;
 * &lt;/h:query&gt;
 *
 *    CREATE OR REPLACE PACKAGE GEP00002 AS
 *        PROCEDURE GEP00002(
 *            P_STATUS    OUT    NUMBER,
 *            P_ERR_CODE  OUT    ERR_MSG_ARRAY,
 *            P_MIDDB     IN     VARCHAR2,
 *            P_USRUPD    IN     VARCHAR2  );
 *    END;
 *
 * @og.group データ表示
 * @og.group データ編集
 *
 * @version  4.0
 * @author   高橋正和
 * @since    JDK5.0,
 */
public class Query_JDBCArrayCallable extends AbstractQuery {
	//* このプログラムのVERSION文字列を設定します。	{@value} */
	private static final String VERSION = "4.0.0 (2005/08/31)" ;

	/**
	 * クエリーを実行します。
	 * セットされているステートメント文字列とそのタイプが合っていない場合は,
	 * エラーになります。
	 * 実行結果は、DBTableModel にセットされます。
	 *
	 */
	public void execute() {
		execute( null );
	}

	/**
	 * 引数配列付のクエリーを実行します。
	 * 処理自体は, #execute() と同様に、各サブクラスの実装に依存します。
	 * これは、PreparedQuery で使用する引数を配列でセットするものです。
	 * select * from emp where deptno = ? and job = ? などの PreparedQuery の
	 * ? 部分の引数を
	 * 順番にセットしていきます。
	 *
	 * @param   args オブジェクトの引数配列
	 */
	public void execute( final String[] args ) {
		CallableStatement callStmt = null ;
		try {
			Connection conn = getConnection();
			callStmt  = getConnection().prepareCall( getStatement() );
			callStmt.setQueryTimeout( DB_MAX_QUERY_TIMEOUT );
			Map<String,Class<?>> map = conn.getTypeMap();
			map.put( ERR_MSG,DBErrMsg.class );

//			ArrayDescriptor sd = ArrayDescriptor.createDescriptor( ARG_ARRAY, conn );
			callStmt.registerOutParameter(1, Types.INTEGER);
			callStmt.registerOutParameter(2, OracleTypes.ARRAY,ERR_MSG_ARRAY);
			if( args != null ) {
				for( int i=0; i<args.length; i++ ) {
					callStmt.setObject( i+3,StringUtil.rTrim( args[i] ) );
				}
			}
			callStmt.execute();

			int rtnCode = callStmt.getInt(1);
			setErrorCode( rtnCode );

			if( rtnCode > ErrorMessage.OK ) {		// 正常以外の場合
				ARRAY rtn3 = ((OracleCallableStatement)callStmt).getARRAY(2);
				Object[] rtnval3 = (Object[])rtn3.getArray();
				ErrorMessage errMessage = new ErrorMessage( "Query_JDBCArrayCallable Error!!" );
				for( int i=0; i<rtnval3.length; i++ ) {
					DBErrMsg er = (DBErrMsg)rtnval3[i];
					if( er == null ) { break; }
					errMessage.addMessage( er.getErrMsg() );
				}
				setErrorMessage( errMessage );
			}
		}
		catch ( SQLException ex ) {
			setErrorCode( ErrorMessage.EXCEPTION );
			String errMsg = ex.getMessage() + ":" + ex.getSQLState() + HybsSystem.CR
						+ getStatement() + HybsSystem.CR;
			rollback();
			realClose();
			throw new HybsSystemException( errMsg,ex );
		}
		finally {
			Closer.stmtClose( callStmt );
		}
	}
}

==> openGion_dev_5.5.6.1.zip: opengion/uap/webapps/gf/src/org/opengion/plugin/query/Query_JDBCArrayCallable.java <==
/*
 * Copyright (c) 2009 The openGion Project.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.opengion.plugin.query;

import org.opengion.hayabusa.common.HybsSystem;
import org.opengion.hayabusa.common.HybsSystemException;
import org.opengion.hayabusa.db.AbstractQuery;
import org.opengion.hayabusa.db.DBErrMsg;
import org.opengion.fukurou.util.ErrorMessage;
import org.opengion.fukurou.util.StringUtil;
import org.opengion.fukurou.util.Closer;

import java.sql.CallableStatement;
import java.sql.Connection;
import java.sql.SQLException;
import java.sql.Types;
import java.util.Map;

import oracle.jdbc.OracleCallableStatement;
import oracle.jdbc.OracleTypes;
import oracle.sql.ARRAY;

/**
 * Callableのエラー配列対応版です。バッチ系標準のPL/SQL をコールする Query クラスです。
 *
 * java.sql.CallableStatement を用いて、データベース検索処理を行います。
 * 引数は、従来のPL/SQLの実行が可能なように、第一引数はエラーコード、第二引数は、
 * エラーメッセージを返してきます。第三引数以降は、自由に指定できます。
 * 内部変数の受け渡しのデフォルト実装は、AbstractQuery クラスを継承している
 * ため,ここでは、execute() メソッドを実装しています。
 *
 * @og.formSample
 * 例：
 *     第一引数、第二引数は、通常のPL/SQLと同じ、結果(STATUS)と
 *     内容(ERR_CODE)を返します。
 *     それ以降の引数については、入力(IN)のみですが、自由に設定できます。
 *     引数に変数を使用する場合は、? 記号を当てはめます。
 *     第一引数、第二引数は、予約済みですが、それ以降は、好きな位置に割り当てられます。
 *     names 属性の順番に、値だけがセットされていきます。
 *     下記の例は、変数の引数は、使用していません。
 *
 * &lt;og:query
 *     command=&quot;NEW&quot;
 *     queryType=&quot;JDBCArrayCallable&quot;
 *     displayMsg=&quot;&quot; &gt;
 *         { call GEP00002.GEP00002( ?,?,&#39;{&#064;GUI.KEY}&#39;,&#39;{&#064;USER.ID}&#39; ) }
 * &lt;/og:query&gt;
 *
 *    CREATE OR REPLACE PACKAGE GEP00002 AS
 *        PROCEDURE GEP00002(
 *            P_STATUS    OUT    NUMBER,
 *            P_ERR_CODE  OUT    ERR_MSG_ARRAY,
 *            P_MIDDB     IN     VARCHAR2,
 *            P_USRUPD    IN     VARCHAR2  );
 *    END;
 *
 * @og.group データ表示
 * @og.group データ編集
 *
 * @version  4.0
 * @author   高橋正和
 * @since    JDK5.0,
 */
public class Query_JDBCArrayCallable extends AbstractQuery {
	//* このプログラムのVERSION文字列を設定します。	{@value} */
	private static final String VERSION = "4.0.0 (2005/08/31)" ;

	/**
	 * クエリーを実行します。
	 * セットされているステートメント文字列とそのタイプが合っていない場合は,
	 * エラーになります。
	 * 実行結果は、DBTableModel にセットされます。
	 *
	 */
	@Override
	public void execute() {
		execute( null );
	}

	/**
	 * 引数配列付のクエリーを実行します。
	 * 処理自体は, #execute() と同様に、各サブクラスの実装に依存します。
	 * これは、PreparedQuery で使用する引数を配列でセットするものです。
	 * select * from emp where deptno = ? and job = ? などの PreparedQuery の
	 * ? 部分の引数を
	 * 順番にセットしていきます。
	 *
	 * @param   args オブジェクトの引数配列
	 */
	@Override
	public void execute( final String[] args ) {
		CallableStatement callStmt = null ;
		try {
			Connection conn = getConnection();
			callStmt  = getConnection().prepareCall( getStatement() );
			callStmt.setQueryTimeout( DB_MAX_QUERY_TIMEOUT );
			Map<String,Class<?>> map = conn.getTypeMap();
			map.put( ERR_MSG,DBErrMsg.class );

//			ArrayDescriptor sd = ArrayDescriptor.createDescriptor( ARG_ARRAY, conn );
			callStmt.registerOutParameter(1, Types.INTEGER);
			callStmt.registerOutParameter(2, OracleTypes.ARRAY,ERR_MSG_ARRAY);
			if( args != null ) {
				for( int i=0; i<args.length; i++ ) {
					callStmt.setObject( i+3,StringUtil.rTrim( args[i] ) );
				}
			}
			callStmt.execute();

			int rtnCode = callStmt.getInt(1);
			setErrorCode( rtnCode );

			if( rtnCode > ErrorMessage.OK ) {		// 正常以外の場合
				ARRAY rtn3 = ((OracleCallableStatement)callStmt).getARRAY(2);
				Object[] rtnval3 = (Object[])rtn3.getArray();
				ErrorMessage errMessage = new ErrorMessage( "Query_JDBCArrayCallable Error!!" );
				for( int i=0; i<rtnval3.length; i++ ) {
					DBErrMsg er = (DBErrMsg)rtnval3[i];
					if( er == null ) { break; }
					errMessage.addMessage( er.getErrMsg() );
				}
				setErrorMessage( errMessage );
			}
		}
		catch ( SQLException ex ) {
			setErrorCode( ErrorMessage.EXCEPTION );
			String errMsg = ex.getMessage() + ":" + ex.getSQLState() + HybsSystem.CR
						+ getStatement() + HybsSystem.CR;
			rollback();
			realClose();
			throw new HybsSystemException( errMsg,ex );
		}
		finally {
			Closer.stmtClose( callStmt );
		}
	}
}
